import json

import pytest

from .resources import searched_observable
from trustar2.models.searched_observable import SearchedObservable
from trustar2.trustar_enums import ObservableTypes

VALUE = "2.2.2.2"
TYPE = ObservableTypes.IP4.value
FIRST_SEEN = 1623273177255
LAST_SEEN = 1623701072520
ENCLAVE_GUIDS = ["test-enclave-guid"]
TAGS = ["test-tag"]


@pytest.fixture
def searched_observable_json():
    return json.loads(searched_observable)


@pytest.fixture
def searched_observable_obj():
    return SearchedObservable(
        value=VALUE,
        type=TYPE,
        first_seen=FIRST_SEEN,
        last_seen=LAST_SEEN,
        enclave_guids=ENCLAVE_GUIDS,
        tags=TAGS
    )


def test_searched_observable_deserialization(searched_observable_json):
    searched_observable = SearchedObservable.from_dict(searched_observable_json)

    assert searched_observable.value == VALUE
    assert searched_observable.type == TYPE
    assert searched_observable.first_seen == FIRST_SEEN
    assert searched_observable.last_seen == LAST_SEEN
    assert searched_observable.enclave_guids == ENCLAVE_GUIDS
    assert searched_observable.tags == TAGS


def test_searched_observable_serialization(searched_observable_obj, searched_observable_json):
    assert searched_observable_obj.serialize() == searched_observable_json


def test_searched_observable_repr(searched_observable_obj):
    assert searched_observable_obj.__repr__() == "SearchedObservable(type=IP4, value=2.2.2.2)"



from __future__ import unicode_literals

from .base import Base
from .entity import Entity
from trustar2.base import fluent
from trustar2.trustar_enums import ObservableTypes, MaxValues


@fluent
class Indicator(Base):

    FIELD_METHOD_MAPPING = {
        "validFrom": "set_valid_from",
        "validTo": "set_valid_to",
        "maliciousScore": "set_malicious_score",
        "confidenceScore": "set_confidence_score",
        "attributes": "_set_attributes_from_dict",
        "relatedObservables": "_set_related_obs_from_dict",
        "tags": "set_tags",
        "properties": "set_properties"
    } 

    def __init__(self, entity_type, value):
        self.observable = Entity(ObservableTypes, entity_type, value, alias="observable")
        self.attributes = []
        self.related_observables = []
        self.tags = []

    def set_related_observables(self, related_obs):
        if isinstance(related_obs, list):
            self.related_observables += related_obs
        else:
            self.related_observables.append(related_obs)

        if len(self.related_observables) > MaxValues.RELATED_OBSERVABLES.value:
            self.related_observables = self.related_observables[:MaxValues.RELATED_OBSERVABLES.value]

    def set_attributes(self, related_attribute):
        if isinstance(related_attribute, list):
            self.attributes += related_attribute
        else:
            self.attributes.append(related_attribute)

        if len(self.attributes) > MaxValues.ATTRIBUTES.value:
            self.attributes = self.attributes[:MaxValues.ATTRIBUTES.value]

    def set_valid_to(self, valid_to):
        if valid_to is not None:
            self.observable.set_valid_to(valid_to)

    def set_valid_from(self, valid_from):
        if valid_from is not None:
            self.observable.set_valid_from(valid_from)

    def set_malicious_score(self, mal_score):
        if mal_score is not None:
            self.observable.set_malicious_score(mal_score)

    def set_confidence_score(self, conf_score):
        if conf_score is not None:
            self.observable.set_confidence_score(conf_score)

    def set_properties(self, properties):
        self.observable.set_properties(properties)

    @property
    def valid_to(self):
        return self.observable.valid_to

    @property
    def valid_from(self):
        return self.observable.valid_from

    @property
    def malicious_score(self):
        return self.observable.malicious_score

    @property
    def confidence_score(self):
        return self.observable.confidence_score

    @property
    def properties(self):
        return self.observable.properties


    def set_tags(self, tag):
        if isinstance(tag, list):
            self.tags += tag
        else:
            self.tags.append(tag)

        if len(self.tags) > MaxValues.TAGS.value:
            self.tags = self.tags[:MaxValues.TAGS.value]

    def serialize(self):
        serialized = {}
        serialized.update(self.observable.serialize())
        serialized.update({"attributes": [attr.serialize() for attr in self.attributes] if len(self.attributes) else []})
        serialized.update({"relatedObservables": [attr.serialize() for attr in self.related_observables] if len(self.related_observables) else []})
        serialized.update({"tags": self.tags})
        return serialized


    def _set_attributes_from_dict(self, attributes):
        if len(attributes) > 0:
            self.set_attributes([Entity.from_dict(a) for a in attributes])


    def _set_related_obs_from_dict(self, observables):
        if len(observables) > 0:
            self.set_related_observables([Entity.from_dict(o) for o in observables])


    @classmethod
    def from_dict(cls, ioc_dict):
        observable = ioc_dict.pop("observable")
        indicator = cls(observable.get("type"), observable.get("value"))
        
        for field, value in ioc_dict.items():
            method_name = cls.FIELD_METHOD_MAPPING.get(field)
            if method_name:
                method = getattr(indicator, method_name)
                method(value)

        return indicator



import json

import pytest

from .resources import prioritized_indicator
from trustar2.models.entity import Entity
from trustar2.models.score_context import ScoreContext
from trustar2.models.prioritized_indicator import PrioritizedIndicator
from trustar2.trustar_enums import ObservableTypes, AttributeTypes, MaliciousScore


IOC_GUID = "test-guid"
ENCLAVE_GUID = "test-enclave-guid"
WORKFLOW_GUID = "test-workflow-guid"
OBS_VALUE = "2.2.2.2"
OBS_TYPE = ObservableTypes.IP4.value
PRIORITY_SCORE = MaliciousScore.HIGH.value
ATTR_VALUE = "MalwareName"
ATTR_TYPE = AttributeTypes.MALWARE.value
USER_TAGS = []
SUBMISSION_TAGS = ["malware"]
CREATED = 1616176082000
UPDATED = 1624986245000
PROCESSED_AT = 1624990135728
SC_ENCLAVE_GUID = "test-score-context-enclave-guid"
SC_SOURCE_NAME = "Test Source"
SC_NORMALIZED_SCORE = 3
SC_WEIGHT = 3.0
SC_PROPERTIES = {}
SC_ENCLAVE_NAME = "Test Source"


@pytest.fixture
def prioritized_indicator_json():
    return json.loads(prioritized_indicator)


@pytest.fixture
def prioritized_indicator_obj():
    return PrioritizedIndicator(
        guid=IOC_GUID,
        enclave_guid=ENCLAVE_GUID,
        workflow_guid=WORKFLOW_GUID,
        observable=Entity.observable(OBS_TYPE, OBS_VALUE),
        priority_score=PRIORITY_SCORE,
        attributes=[Entity.attribute(ATTR_TYPE, ATTR_VALUE)],
        user_tags=USER_TAGS,
        submission_tags=SUBMISSION_TAGS,
        score_contexts=[ScoreContext(
            enclave_guid=SC_ENCLAVE_GUID,
            source_name=SC_SOURCE_NAME,
            normalized_score=SC_NORMALIZED_SCORE,
            weight=SC_WEIGHT,
            properties=SC_PROPERTIES,
            enclave_name=SC_ENCLAVE_NAME
        )],
        created=CREATED,
        updated=UPDATED,
        processed_at=PROCESSED_AT,
        safelisted=False
    )


def test_prioritized_indicator_deserialization(prioritized_indicator_json):
    prioritized_ioc = PrioritizedIndicator.from_dict(prioritized_indicator_json)

    assert prioritized_ioc.guid == IOC_GUID
    assert prioritized_ioc.enclave_guid == ENCLAVE_GUID
    assert prioritized_ioc.workflow_guid == WORKFLOW_GUID
    assert prioritized_ioc.observable.value == OBS_VALUE
    assert prioritized_ioc.observable.type == OBS_TYPE
    assert prioritized_ioc.priority_score == PRIORITY_SCORE
    assert prioritized_ioc.attributes[0].value == ATTR_VALUE
    assert prioritized_ioc.attributes[0].type == ATTR_TYPE
    assert prioritized_ioc.user_tags == USER_TAGS
    assert prioritized_ioc.submission_tags == SUBMISSION_TAGS
    assert prioritized_ioc.score_contexts[0].enclave_guid == SC_ENCLAVE_GUID
    assert prioritized_ioc.score_contexts[0].source_name == SC_SOURCE_NAME
    assert prioritized_ioc.score_contexts[0].weight == SC_WEIGHT
    assert prioritized_ioc.score_contexts[0].normalized_score == SC_NORMALIZED_SCORE
    assert prioritized_ioc.score_contexts[0].enclave_name == SC_ENCLAVE_NAME
    assert prioritized_ioc.score_contexts[0].properties == SC_PROPERTIES
    assert prioritized_ioc.created == CREATED
    assert prioritized_ioc.updated == UPDATED
    assert prioritized_ioc.processed_at == PROCESSED_AT
    assert not prioritized_ioc.safelisted


def test_prioritized_indicator_serialization(prioritized_indicator_obj, prioritized_indicator_json):
    assert prioritized_indicator_obj.serialize() == prioritized_indicator_json


def test_prioritized_indicator_repr(prioritized_indicator_obj):
    assert prioritized_indicator_obj.__repr__() == "PrioritizedIndicator(type=IP4, value=2.2.2.2)"



# Seems like from time to time people do a from trustar2 import *, lets minimize the chances of something ugly happening
__all__ = ["get_logger", "TrustarJSONFormatter"]

import datetime
import logging
import logging.handlers
import os
import sys

import json_log_formatter

from .config import (
    LOGGING_LEVEL_VAR,
    LOGGING_SHOW_LEVEL_VAR,
    LOGGING_SHOW_MODULE_VAR,
    LOGGING_SHOW_TIME_VAR,
    LOGGING_SHOW_STASH_INFO_VAR,
    LOGGING_FILENAME_VAR,
    LOGGING_STDOUT_ENABLED_VAR,
)

DEFAULT_LOGGING_LEVEL = logging.INFO

class TrustarJSONFormatter(json_log_formatter.JSONFormatter):
    """
    Custom class to override the default behaviour of the JSONFormatter
    """

    def __init__(self, stash_config=None):
        super(TrustarJSONFormatter, self).__init__()
        self._stash_config = stash_config

    def format(self, record):
        """
        the default behaviour of JSONFormatter is to cast everything into a string
        we avoid calling getMessage and leave the object as it is, this way our json messages can have nested dicts
        """
        message = record.msg
        extra = self.extra_from_record(record)
        json_record = self.json_record(message, extra, record)
        # Backwards compatibility: Functions that overwrite this but don't
        # return a new value will return None because they modified the
        # argument passed in. --v
        mutated_record = self.mutate_json_record(json_record) or json_record
        return self.to_json(mutated_record)

    def json_record(self, message, extra, record):
        extra['message'] = message

        extra = self._add_system_logs_to_json(extra, record)
        extra = self._add_stash_logs_to_json(extra)
        if record.exc_info:
            extra['exec_info'] = self.formatException(record.exc_info)
        return extra

    def _add_stash_logs_to_json(self, extra):
        if not int(os.environ.get(LOGGING_SHOW_STASH_INFO_VAR, 1)):
            return extra
        if not self._stash_config:
            return extra
        
        extra['stash_type'] = self._stash_config.get("stash_type", "UNKNOWN")
        extra['stash_name'] = self._stash_config.get("stash_name", 'UNKNOWN')
        extra['stash_id'] = self._stash_config.get("stash_id", 'UNKNOWN')
        extra['enclave_ids'] = self._stash_config.get("enclave_ids", 'UNKNOWN')

        return extra

    def _add_system_logs_to_json(self, extra, record):
        if int(os.environ.get(LOGGING_SHOW_LEVEL_VAR, 1)):
            extra['level'] = record.levelname
        
        if int(os.environ.get(LOGGING_SHOW_MODULE_VAR, 1)):
            extra['module'] = record.name
        
        if int(os.environ.get(LOGGING_SHOW_TIME_VAR, 1)):
            extra['time'] = datetime.datetime.utcnow()
        
        return extra

    def to_json(self, record):
        """Converts record dict to a JSON string.

        It makes best effort to serialize a record (represents an object as a string)
        instead of raising TypeError if json library supports default argument.
        Note, ujson doesn't support it.
        """
        return self.json_lib.dumps(record, default=_json_object_encoder)


def _json_object_encoder(obj):
    try:
        return obj.to_json()
    except AttributeError:
        return str(obj)


def get_stdout_handler(formatter):
    """
    Gets the handler to manage the output of the logger, default: stdout
    """
    handler = logging.StreamHandler(sys.stdout)
    handler.setFormatter(formatter)
    return handler

def get_file_handler(filename, formatter):
    handler = logging.handlers.RotatingFileHandler(
                filename=filename,
                mode='a',
                maxBytes=8192000,
                backupCount=10,
        )
    handler.setFormatter(formatter)
    return handler


def get_formatter(stash_config=None):
    return TrustarJSONFormatter(stash_config=stash_config)

def get_log_file(filename=None):
    """
        Return the log file filename if its set.
        It also create the file if it does not exist.
    """
    filename = filename or os.environ.get(LOGGING_FILENAME_VAR)
    if not filename:
        return None
    with open(filename, "a+"):
        return filename

def get_logger(
                name=None,
                stash_config=None,
                filename=None,
                stdout_enabled=True,
                level=None
            ):
    logger = logging.getLogger(name or __name__)

    stdout_enabled = stdout_enabled and int(os.environ.get(LOGGING_STDOUT_ENABLED_VAR, 1))
    if stdout_enabled:
        stdout_formatter = get_formatter(stash_config=stash_config)
        stdout_handler = get_stdout_handler( formatter=stdout_formatter)
        logger.addHandler(stdout_handler)
    
    file = get_log_file(filename)
    if file:
        file_formatter = get_formatter(stash_config=stash_config)
        file_handler = get_file_handler(file, formatter=file_formatter)
        logger.addHandler(file_handler)

    level = level or int(os.environ.get(LOGGING_LEVEL_VAR, DEFAULT_LOGGING_LEVEL))
    logger.setLevel(level)
    return logger



import json

import pytest

from .resources import searched_submission
from trustar2.models.searched_submission import SearchedSubmission


TITLE = "Test Submission Title"
GUID = "test-guid"
ENCLAVE_GUID = "test-enclave-guid"
CREATED = 1624980621003
UPDATED = 1624980621003
TAGS = []


@pytest.fixture
def searched_submission_obj():
    return SearchedSubmission(
        guid=GUID,
        enclave_guid=ENCLAVE_GUID,
        title=TITLE,
        created=CREATED,
        updated=UPDATED,
        tags=TAGS
    )


@pytest.fixture
def searched_submission_json():
    return json.loads(searched_submission)


def test_searched_submission_deserialization(searched_submission_json):
    searched_submission = SearchedSubmission.from_dict(searched_submission_json)

    assert searched_submission.title == TITLE
    assert searched_submission.guid == GUID
    assert searched_submission.enclave_guid == ENCLAVE_GUID
    assert searched_submission.created == CREATED
    assert searched_submission.updated == UPDATED
    assert searched_submission.tags == TAGS


def test_searched_submission_serialization(searched_submission_obj, searched_submission_json):
    assert searched_submission_obj.serialize() == searched_submission_json


def test_searched_submission_repr(searched_submission_obj):
    assert searched_submission_obj.__repr__() == "SearchedSubmission(title={})".format(TITLE)



from trustar2.base import typename
from trustar2.models import Entity
from trustar2.models.base import Base
from trustar2.models.score_context import ScoreContext

class PrioritizedIndicator(Base):

    def __init__(self, guid, enclave_guid, workflow_guid, observable, priority_score, attributes, 
                 user_tags, submission_tags, score_contexts, created, updated, processed_at, safelisted):

        self.guid = guid
        self.enclave_guid = enclave_guid
        self.workflow_guid = workflow_guid
        self.observable = observable
        self.priority_score = priority_score
        self.attributes = attributes
        self.user_tags = user_tags
        self.submission_tags = submission_tags
        self.score_contexts = score_contexts
        self.created = created
        self.updated = updated
        self.processed_at = processed_at
        self.safelisted = safelisted


    def __repr__(self):
        return "{}(type={}, value={})".format(typename(self), self.observable.type, self.observable.value)


    @classmethod
    def from_dict(cls, ioc_dict):
        observable = ioc_dict.get("observable")
        observable_obj = Entity.observable(
            observable.get("type"), 
            observable.get("value")
        )

        attributes = [
            Entity.attribute(e.get("type"), e.get("value"))
            for e in ioc_dict.get("attributes")
        ]

        score_contexts = [
            ScoreContext.from_dict(sc) 
            for sc in ioc_dict.get("scoreContexts")
        ]

        return cls(
            guid=ioc_dict.get("guid"),
            enclave_guid=ioc_dict.get("enclaveGuid"),
            workflow_guid=ioc_dict.get("workflowGuid"),
            observable=observable_obj,
            priority_score=ioc_dict.get("priorityScore"),
            attributes=attributes,
            user_tags=ioc_dict.get("userTags"),
            submission_tags=ioc_dict.get("submissionTags"),
            score_contexts=score_contexts,
            created=ioc_dict.get("created"),
            updated=ioc_dict.get("updated"),
            processed_at=ioc_dict.get("processedAt"),
            safelisted=ioc_dict.get("safelisted")
        )


    def serialize(self):
        serialized = super(PrioritizedIndicator, self).serialize()
        observable = serialized.get("observable").get("entity")
        attributes = [a.get("entity") for a in serialized.get("attributes")]
        serialized.update({"observable": observable, "attributes": attributes})
        return serialized



from trustar2.models.base import Base


class ScoreContext(Base):

    def __init__(self, enclave_guid, source_name, normalized_score, weight, properties, enclave_name): 
        self.enclave_guid = enclave_guid
        self.source_name = source_name
        self.normalized_score = normalized_score
        self.weight = weight
        self.properties = properties
        self.enclave_name = enclave_name


    @classmethod
    def from_dict(cls, score_dict):
        return cls(
            enclave_guid=score_dict.get("enclaveGuid"),
            source_name=score_dict.get("sourceName"),
            normalized_score=score_dict.get("normalizedScore"),
            weight=score_dict.get("weight"),
            properties=score_dict.get("properties"),
            enclave_name=score_dict.get("enclaveName")
        )



from .api_client import ApiClient
from .base import Param, fluent

from trustar2.models.trustar_response import TruStarResponse
from trustar2.models.searched_observable import SearchedObservable
from trustar2.models.searched_submission import SearchedSubmission
from trustar2.models.prioritized_indicator import PrioritizedIndicator

ENDPOINT_IDX = -2


@fluent
class Query:

    def __init__(self, config, endpoint, method, params=None, query_string=None):
        self.config = config
        self.endpoint = endpoint
        self.method = method
        self.params = params
        self.query_string = query_string
        self.stop = False
        self.api = ApiClient(self.config)

    def __iter__(self):
        return self

    def __next__(self):
        return self.next()

    def set_query_string(self, query_string):
        self.query_string = query_string

    def set_params(self, params):
        self.params = params

    def _update_cursor(self, response):
        if response["responseMetadata"].get("nextCursor", "") != "":
            cursor = Param("cursor", response["responseMetadata"]["nextCursor"])
            self.params.add(cursor)
        else:
            self.stop = True

    def _update_page(self, response):
        if response["hasNext"]:
            currentPage = int(response.get("pageNumber", 0))
            pageNumber = Param("pageNumber", currentPage + 1)
            self.params.add(pageNumber)
        else:
            self.stop = True

    def _update_params_from_response(self, response):
        if "responseMetadata" in response.keys():
            self._update_cursor(response)
        elif "hasNext" in response.keys():
            self._update_page(response)


    def _get_content_from_endpoint(self, result):
        endpoint_obj = {
            "indicators": PrioritizedIndicator, 
            "observables": SearchedObservable, 
            "submissions": SearchedSubmission
        }
        endpoint = self.endpoint.rsplit("/")[ENDPOINT_IDX]
        obj = endpoint_obj.get(endpoint)
        return [obj.from_dict(i) for i in result.json().get("items")]


    def next(self):
        if not self.stop:
            result = self.api.fetch(self, use_empty_payload=True)
            self._update_params_from_response(result.json())

            return TruStarResponse(
                status_code = result.status_code,
                data=self._get_content_from_endpoint(result)
            )

        else:
            raise StopIteration

    def execute(self, use_empty_payload=False):
        return self.api.fetch(self, use_empty_payload)



from trustar2.base import typename
from trustar2.models.base import Base

class SearchedSubmission(Base):

    def __init__(self, guid, enclave_guid, title, created, updated, tags):
        self.guid = guid
        self.enclave_guid = enclave_guid
        self.title = title
        self.created = created
        self.updated = updated
        self.tags = tags


    def __repr__(self):
        return "{}(title={})".format(typename(self), self.title)


    @classmethod
    def from_dict(cls, sub_dict):
        return cls(
            guid=sub_dict.get("guid"),
            enclave_guid=sub_dict.get("enclaveGuid"),
            title = sub_dict.get("title"),
            created=sub_dict.get("created"),
            updated=sub_dict.get("updated"),
            tags=sub_dict.get("tags")
        )



LOGGING_LEVEL_VAR = "TRUSTAR_LOGGING_LEVEL"
LOGGING_SHOW_TIME_VAR = "TRUSTAR_LOGGING_SHOW_TIME"
LOGGING_SHOW_MODULE_VAR = "TRUSTAR_LOGGING_SHOW_MODULE"
LOGGING_SHOW_LEVEL_VAR = "TRUSTAR_LOGGING_SHOW_LEVEL"
LOGGING_SHOW_STASH_INFO_VAR = "TRUSTAR_LOGGING_SHOW_STASH_INFO"
LOGGING_FILENAME_VAR = "TRUSTAR_LOGGING_FILENAME"
LOGGING_STDOUT_ENABLED_VAR = "TRUSTAR_LOGGING_STDOUT_ENABLED"



from trustar2.base import typename
from trustar2.models.base import Base
from trustar2.models.indicator import Indicator


class SubmissionDetails(Base):

    def __init__(self, id, title, content, enclave_guid, external_id, external_url, 
                 timestamp, created, updated, tags, submission_version):

        self.id = id
        self.title = title
        self.content = content
        self.enclave_guid = enclave_guid
        self.external_id = external_id
        self.external_url = external_url
        self.timestamp = timestamp
        self.created = created
        self.updated = updated
        self.tags = tags
        self.submission_version = submission_version


    def __repr__(self):
        return "{}(title={})".format(typename(self), self.title)



class StructuredSubmissionDetails(SubmissionDetails):

    def __init__(self, id, title, content, enclave_guid, external_id, external_url, 
                 timestamp, created, updated, tags, submission_version, raw_content):

        content_indicators = [Indicator.from_dict(i) for i in content.get("indicators")]
        content.update({"indicators": content_indicators})
        super(StructuredSubmissionDetails, self).__init__(
            id, title, content, enclave_guid, 
            external_id, external_url, timestamp, 
            created, updated, tags, submission_version
        )
        self.raw_content = raw_content


    @classmethod
    def from_dict(cls, sub_dict):
        return cls(
            id=sub_dict.get("id"),
            title=sub_dict.get("title"),
            content=sub_dict.get("content"),
            enclave_guid=sub_dict.get("enclaveGuid"),
            external_id=sub_dict.get("externalId"),
            external_url=sub_dict.get("externalUrl"),
            timestamp=sub_dict.get("timestamp"),
            created=sub_dict.get("created"),
            updated=sub_dict.get("updated"),
            tags=sub_dict.get("tags"),
            submission_version=sub_dict.get("submissionVersion"),
            raw_content=sub_dict.get("rawContent")
        )


    def serialize(self):
        serialized = super(StructuredSubmissionDetails, self).serialize()
        content_indicators = [i.serialize() for i in serialized.get("content").get("indicators")]
        serialized.update({"content": {"indicators": content_indicators}})
        return serialized



class UnstructuredSubmissionDetails(SubmissionDetails):

    def __init__(self, id, title, content, enclave_guid, external_id, external_url, 
                 timestamp, created, updated, tags, submission_version):

        super(UnstructuredSubmissionDetails, self).__init__(
            id, title, content, enclave_guid, 
            external_id, external_url, timestamp, 
            created, updated, tags, submission_version
        )


    @classmethod
    def from_dict(cls, sub_dict):
        return cls(
            id=sub_dict.get("id"),
            title=sub_dict.get("title"),
            content=sub_dict.get("content"),
            enclave_guid=sub_dict.get("enclaveGuid"),
            external_id=sub_dict.get("externalId"),
            external_url=sub_dict.get("externalUrl"),
            timestamp=sub_dict.get("timestamp"),
            created=sub_dict.get("created"),
            updated=sub_dict.get("updated"),
            tags=sub_dict.get("tags"),
            submission_version=sub_dict.get("submissionVersion")
        )



from trustar2.base import typename
from trustar2.models.base import Base

class SearchedObservable(Base):

    def __init__(self, type, value, first_seen, last_seen, enclave_guids, tags):
        self.type = type
        self.value = value
        self.first_seen = first_seen
        self.last_seen = last_seen
        self.enclave_guids = enclave_guids
        self.tags = tags


    def __repr__(self):
        return "{}(type={}, value={})".format(typename(self), self.type, self.value)


    @classmethod
    def from_dict(cls, obs_dict):
        return cls(
            type=obs_dict.get("type"),
            value=obs_dict.get("value"),
            first_seen=obs_dict.get("firstSeen"),
            last_seen=obs_dict.get("lastSeen"),
            enclave_guids=obs_dict.get("enclaveGuids"),
            tags=obs_dict.get("tags")
        )



from __future__ import unicode_literals

import json

import pytest

from trustar2.models import Indicator, Entity
from .resources import indicators_submission_example_request
from trustar2.trustar_enums import (
    ObservableTypes, AttributeTypes, 
    ConfidenceScore, MaliciousScore
)


URL = ObservableTypes.URL.value
IP4 = ObservableTypes.IP4.value
EMAIL_ADDRESS = ObservableTypes.EMAIL_ADDRESS.value
THREAT_ACTOR = AttributeTypes.THREAT_ACTOR.value
MALWARE = AttributeTypes.MALWARE.value

FROM_TIMESTAMP = 1604510497000
TO_TIMESTAMP = 1607102497000

LOW_CONFIDENCE = ConfidenceScore.LOW.value
MEDIUM_CONFIDENCE = ConfidenceScore.MEDIUM.value
HIGH_CONFIDENCE = ConfidenceScore.HIGH.value
BENIGN_SCORE = MaliciousScore.BENIGN.value


@pytest.fixture
def indicator():
    return Indicator(URL, "www.badurl.com")


@pytest.fixture
def indicator_json():
    return json.loads(indicators_submission_example_request).get("content").get("indicators")[0]


def test_set_related_observables(indicator):
    related_observable_email = Entity.observable(EMAIL_ADDRESS, "bob@gmail.com")
    indicator.set_related_observables(related_observable_email)
    assert related_observable_email in indicator.related_observables


def test_set_attributes(indicator):
    attributes = Entity.attribute(THREAT_ACTOR, "ActorName")
    indicator.set_attributes(attributes)
    assert attributes in indicator.attributes


def test_set_valid_to(indicator):
    indicator.set_valid_to(1604510497)
    value = indicator.observable.params.get("validTo")
    assert value == 1604510497


def test_set_valid_from(indicator):
    indicator.set_valid_from(1604510497)
    value = indicator.observable.params.get("validFrom")
    assert value == 1604510497


def test_set_malicious_score(indicator):
    indicator.set_malicious_score("HIGH")
    value = indicator.observable.params.get("maliciousScore")
    assert value == "HIGH"


def test_set_confidence_score(indicator):
    indicator.set_confidence_score("HIGH")
    value = indicator.observable.params.get("confidenceScore")
    assert value == "HIGH"

 
def test_set_properties(indicator):
    properties = {"myPropKey": "myPropValue"}
    indicator.set_properties(properties)
    value = indicator.observable.params.get("properties")
    assert value == properties


def test_set_properties_with_more_than_twenty_elements(indicator):
    properties = {"prop" + str(i): "value" + str(i) for i in range(21)}
    with pytest.raises(AttributeError):
        indicator.set_properties(properties)


def test_set_properties_with_element_type_not_a_string(indicator):
    properties = {"myProperty": 100}
    with pytest.raises(AttributeError):
        indicator.set_properties(properties)


def test_set_tags(indicator):
    tag = ["importantTag"]
    indicator.set_tags(tag)
    assert "importantTag" in indicator.tags


def test_indicator_deserialization(indicator_json):
    indicator = Indicator.from_dict(indicator_json)
    assert indicator.observable.type == URL
    assert indicator.observable.value == "verybadurl"
    assert indicator.confidence_score == LOW_CONFIDENCE
    assert indicator.malicious_score == BENIGN_SCORE
    assert indicator.properties == {"propertyKey": "propertyValue"}
    assert indicator.valid_from == FROM_TIMESTAMP
    assert indicator.valid_to == TO_TIMESTAMP

    assert indicator.attributes[0].type == THREAT_ACTOR
    assert indicator.attributes[0].value == "ActorName"
    assert indicator.attributes[0].valid_from == FROM_TIMESTAMP
    assert indicator.attributes[0].valid_to == TO_TIMESTAMP
    assert indicator.attributes[0].confidence_score == LOW_CONFIDENCE
    
    assert indicator.attributes[1].type == MALWARE
    assert indicator.attributes[1].value == "MalwareName"
    assert indicator.attributes[1].valid_from == FROM_TIMESTAMP
    assert indicator.attributes[1].valid_to == TO_TIMESTAMP
    assert indicator.attributes[1].confidence_score == MEDIUM_CONFIDENCE

    assert indicator.related_observables[0].type == IP4
    assert indicator.related_observables[0].value == "2.2.2.2"
    assert indicator.related_observables[0].valid_from == FROM_TIMESTAMP
    assert indicator.related_observables[0].valid_to == TO_TIMESTAMP
    assert indicator.related_observables[0].confidence_score == LOW_CONFIDENCE

    assert indicator.related_observables[1].type == URL
    assert indicator.related_observables[1].value == "wwww.relatedUrl.com"
    assert indicator.related_observables[1].valid_from == FROM_TIMESTAMP
    assert indicator.related_observables[1].valid_to == TO_TIMESTAMP
    assert indicator.related_observables[1].confidence_score == HIGH_CONFIDENCE

    assert indicator.tags == ["importantTag", "anotherTag"]



from __future__ import unicode_literals

from .base import Base
from trustar2.base import fluent, ParamsSerializer, Param, get_timestamp
from trustar2.trustar_enums import AttributeTypes, ObservableTypes



@fluent
class Entity(Base):

    FIELD_METHOD_MAPPING = {
        "validFrom": "set_valid_from",
        "validTo": "set_valid_to",
        "confidenceScore": "set_confidence_score",
    } 
    VALID_TYPES = list(AttributeTypes.members()) + list(ObservableTypes.members())


    def __init__(self, validator, entity_type, value, alias='entity'):
        self.params = ParamsSerializer()
        self.validator = validator
        if entity_type not in validator.members():
            raise AttributeError(
                "Attribute type should be in the following: {}".format(
                    list(validator.members())
                ))
        self.key = alias
        self.set_custom_param(self.key, {"value": value, "type": entity_type})


    def __repr__(self):
        entity = "Observable" if isinstance(self.validator, ObservableTypes) else "Attribute"
        return "{}(type={}, value={})".format(entity, self.type, self.value)


    @classmethod
    def attribute(cls, entity_type, value):
        return Entity(AttributeTypes, entity_type, value)


    @classmethod
    def observable(cls, entity_type, value):
        return Entity(ObservableTypes, entity_type, value)


    @property
    def type(self):
        return self.params.get(self.key).get("type")


    @property
    def value(self):
        return self.params.get(self.key).get("value")


    @property
    def valid_to(self):
        return self.params.get("validTo")


    @property
    def valid_from(self):
        return self.params.get("validFrom")


    @property
    def malicious_score(self):
        return self.params.get("maliciousScore")


    @property
    def confidence_score(self):
        return self.params.get("confidenceScore")


    @property
    def properties(self):
        return self.params.get("properties")


    def set_valid_from(self, valid_from):
        if valid_from is not None:
            valid_from = get_timestamp(valid_from) if not isinstance(valid_from, int) else valid_from
            self.set_custom_param("validFrom", valid_from)


    def set_valid_to(self, valid_to):
        if valid_to is not None:
            valid_to = get_timestamp(valid_to) if not isinstance(valid_to, int) else valid_to
            self.set_custom_param("validTo", valid_to)


    def set_confidence_score(self, confidence_score):
        if confidence_score is not None:
            self.set_custom_param("confidenceScore", confidence_score)


    def set_malicious_score(self, malicious_score):
        self.set_custom_param("maliciousScore", malicious_score)


    def set_properties(self, properties):
        if len(properties) > 20:
            raise AttributeError("properties field can not have more than 20 elements")

        for k, v in properties.items():
            if not isinstance(k, type("")) or not isinstance(v, type("")): # py2 / py3 compatibility
                raise AttributeError("Both key and value of the properties should be strings.")

        self.set_custom_param("properties", properties)


    def set_custom_param(self, key, value):
        param = Param(key=key, value=value)
        self.params.add(param)


    def serialize(self):
        return self.params.serialize()


    @classmethod
    def _get_entity_obj(cls, entity_type, entity):
        return (
            cls.attribute(entity_type, entity.get("value"))
            if entity_type in AttributeTypes.members()
            else cls.observable(entity_type, entity.get("value"))
        )


    @classmethod
    def from_dict(cls, entity_dict):
        entity = entity_dict.pop("entity")
        entity_type = entity.get("type")
        if entity_type not in cls.VALID_TYPES:
            raise AttributeError("Entity type does not correspond to a valid entity type")

        entity_obj = cls._get_entity_obj(entity_type, entity)
        for field, value in entity_dict.items():
            method_name = cls.FIELD_METHOD_MAPPING.get(field)
            if method_name:
                method = getattr(entity_obj, method_name)
                method(value)

        return entity_obj
